// main.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include "PIPMasq.h"
#include "packet32.h"
#include <signal.h>
#include <windows.h>
#include <windowsx.h>

CRITICAL_SECTION	critSec_PacketGetNetInfoEx;
CRITICAL_SECTION	critSec_getAdapterInfo2;
CRITICAL_SECTION	critSec_getAdapterInfo;

bool
rev_inet_addr(__u32 netByteIPAddr32, char *dotIP);

bool getAdapterInfo2(
	const char* nicName,		// input
	char* AdapterName,			// output
	unsigned char*	macAddr,	// output	
	char* ipAddress,			// output				
	char* maskAddress			// output		
);

BOOLEAN PacketGetNetInfoEx(
	LPTSTR AdapterName, PULONG netp, PULONG maskp, PULONG defp, bool *dhcpFlag);

unsigned int __stdcall _outWardThread(void *wardInfo)
{
	struct wardInfo *wi = (struct wardInfo *)wardInfo;

	COutward _outWard(	wi->tableMgr,
						wi->portMgr,
						wi->arpMgr,
						wi->psuedIF,
						wi->threadMgr,
						wi->ftpMgr,
						wi->lanIPAddr,
						wi->wanIPAddr,
						wi->subWanIPAddr,
						wi->defaultWanGWAddr,
						wi->lanNicName,
						wi->wanNicName
						);


	_outWard.start();

	return(0);

}

unsigned int __stdcall _inWardThread(void *wardInfo)
{
	struct wardInfo *wi = (struct wardInfo *)wardInfo;

	CInward _inWard(	wi->tableMgr,
						wi->portMgr,
						wi->arpMgr,
						wi->pingSvr,
						wi->psuedIF,
						wi->threadMgr,
						wi->lanIPAddr,
						wi->wanIPAddr,
						wi->pingDown,
						wi->lanNicName,
						wi->wanNicName, 
						
						wi->dhcpcdMgr
						);

	try
	{
		_inWard.start();
	}
	catch(panic panic) 
	{
		int i=0;
		// revist
	}

	return(0);

}

unsigned int __stdcall _timeMgrThread(void *wardInfo)
{
	struct wardInfo *wi = (struct wardInfo *)wardInfo;

	TimeMgr _timeMgr(	wi->tableMgr,
						wi->threadMgr);

	_timeMgr.start();

	return(0);

}

unsigned int __stdcall _consoleMgrThread(void *wardInfo)
{
	struct wardInfo *wi = (struct wardInfo *)wardInfo;

	ConsoleMgr _consoleMgr(	wi->tableMgr,
							wi->threadMgr,
														
							wi->pseudIPAddr,
							wi->lanIPAddr,
							wi->wanIPAddr,
							wi->subWanIPAddr,
							wi->defaultWanGWAddr,
							wi->pingDown
							);

	_consoleMgr.start();

	return(0);

}



struct wardInfo wi;

bool getDefaultGWMacAddr(
	char* nicName, 
	char* defaultWanGWAddr,
	unsigned char*	defaultGWMacAddr)
{
	CArp	_carp(nicName, defaultWanGWAddr);

	if (_carp.isArpVaild == false)
		return false;

	__u32 defaultWanGWAddr_u32 = inet_addr(defaultWanGWAddr);

	if (_carp.arpSendRecv(defaultWanGWAddr_u32, (char*)defaultGWMacAddr) == false)
		return false;

	return true;
}


bool
loadNetInfo(
			char* pseudIPAddr,
			char* wanNicName,
			char* subWanIPAddr,
			char* defaultWanGWAddr,
			char* lanNicName,
			bool* pingDown
			)
{
	// You should prepare the configuration file that is named "pipmasq.cfg"
	//		
	//	PSEUD=15.2.117.199
	//	OUTERDEV="\\Device\\Packet_PCNTN4M1"
	//	INTRADEV="\\Device\\Packet_FA311ND43"

	memset(pseudIPAddr, 0, 16);
	memset(wanNicName, 0, 255);
	memset(subWanIPAddr, 0, 16);
	memset(defaultWanGWAddr, 0, 16);
	memset(lanNicName, 0, 255);
	
	FILE *f;
	char string[600];
	char buffer[MAX_PATH];
	int counter = 0;
	
	if (GetCurrentDirectory(MAX_PATH, buffer) == 0)
		return false;
	
	char configFileName[MAX_PATH];
	strcpy(configFileName, buffer);
	strcat(configFileName, "\\pipmasq.cfg");
	
	f = fopen(configFileName,"r");
	if (!f)
	{
		printf("[Error] ");
		strcat(configFileName, " is not found.\n");
		printf("%s\n", configFileName);
		return false;
	}
	
	while((string[counter++] = fgetc(f)) != EOF);
	
	counter--;
	string[counter] = NULL;
	
	fclose(f);
	
	bool pseudIPAddr_flag = false;
	bool wanNicName_flag = false;
	bool subWanIPAddr_flag = false;
	bool defaultWanGWAddr_flag = false;
	bool lanNicName_flag = false;
	
	char seps[]   = "= \t\n";
	char *token;
	
	token = strtok( string, seps );
	
	while( token != NULL )
	{
		if (strcmp(token,"PSEUD") == 0)
		{
			if ((token = strtok( NULL, seps )) == NULL)
			{
				printf("[Error] No PSEUD entry in pipmasq.cfg.\n");
				return false;
			}
			
			strcpy(pseudIPAddr, token);
			
			pseudIPAddr_flag = true;
		}
		else if (strcmp(token,"OUTERDEV") == 0)
		{
			if ((token = strtok( NULL, seps )) == NULL)
			{
				printf("[Error] No OUTERDEV entry in pipmasq.cfg.\n");
				return false;
			}
			
			strcpy(wanNicName, token);
			
			wanNicName_flag = true;
		}
		else if (strcmp(token,"INTRADEV") == 0)
		{
			if ((token = strtok( NULL, seps )) == NULL)
			{
				printf("[Error] No INTRADEV entry in pipmasq.cfg.\n");
				return false;
			}
			
			strcpy(lanNicName, token);
			
			lanNicName_flag = true;
		}
		else if (strcmp(token,"PINGDOWN") == 0)
		{
			if ((token = strtok( NULL, seps )) == NULL)
				return false;
			
			if ((strcmp(token,"ON") == 0)|| (strcmp(token,"on") == 0))
				*pingDown = true;
		}
		token = strtok( NULL, seps );
	}
	
	// Get both subWanIPAddr and defaultWanGWAddr using by wanNicName
	ULONG netp, maskp,defp;
	bool dhcpFlag;

	if (PacketGetNetInfoEx(
		(char*)wanNicName, 
		(PULONG)(&netp), 
		(PULONG)(&maskp), 
		(PULONG)(&defp), 
		(bool*)&dhcpFlag) == TRUE)
	{
		rev_inet_addr(maskp, subWanIPAddr);
		rev_inet_addr(defp, defaultWanGWAddr);

		subWanIPAddr_flag = defaultWanGWAddr_flag = true;
	}


	if (pseudIPAddr_flag 
		& wanNicName_flag 
		& subWanIPAddr_flag 
		& defaultWanGWAddr_flag 
		& lanNicName_flag)
	{
		return true;
	}
	else 
	{
		printf("[Error] Syntax error in pipmasq.cfg\n");
		return false;
	}
}	

void
openingMessage(void)
{
	cout <<"==========================================================="<< endl;
	cout <<"                                                           "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"            Pseud IP Masquerade on Win32                   "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"                                       Ver " << VERSION     << endl;
	cout <<"                                                           "<< endl;
	cout <<"                                       Tomoichi Ebata      "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"        http://www.ff.iij4u.or.jp/~ebata/soft/pipmasq/     "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"        This product includes software developed by        "<< endl;
	cout <<"      the Politecnico di Torino, and its contributors.     "<< endl; 
	cout <<"                                                           "<< endl;
	cout <<"          http://netgroup-serv.polito.it/winpcap/          "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"                                                           "<< endl;
	cout <<"==========================================================="<< endl;
}

void _sigCatch(int sig)
{
	printf("\n'Ctrl-c' is not available now. Try 'exit' \n");
	signal(SIGINT,_sigCatch);
}


#if _NTS
DWORD ServiceThread(LPDWORD param)  // This is for NT Service
#else
int main(void) 
#endif
{
	// Multipule start check
	HANDLE hMutex;
	hMutex = CreateMutex(NULL, TRUE, "DSC4PIPMasq");
	
	// If GetLastError() returns ERROR_ALREADY_EXISTS,
	// another PIPMasq has already started.
	if ( (!hMutex) || (GetLastError() == ERROR_ALREADY_EXISTS) )
	{
		openingMessage();
		printf("\n\n");
		printf("Another PIPMasq has already started, or an error has happened.\n");
		printf("\n");
		printf("Existing");

		// Waiting time to show the above message to user
		for (int i=0; i<4; i++)
		{
			printf(".");
			Sleep(500);
		}

		return -1;
	}

	signal(SIGINT,_sigCatch);

	// Set critical section
	InitializeCriticalSection(&critSec_PacketGetNetInfoEx);
	InitializeCriticalSection(&critSec_getAdapterInfo2);
	InitializeCriticalSection(&critSec_getAdapterInfo);

	openingMessage();

	char		lanNicName[512];
	char		wanNicName[512];
	char		subWanIPAddr[16];
	char		defaultWanGWAddr[16];
	char		pseudIPAddr[16];
	bool 		pingDown = false;


	if (!loadNetInfo(
			pseudIPAddr, 
			wanNicName, 
			subWanIPAddr, 
			defaultWanGWAddr, 
			lanNicName,
			&pingDown
			))
		return -1;

	unsigned char	defaultGWMacAddr[ETH_ALEN];

	if (getDefaultGWMacAddr(
		wanNicName, defaultWanGWAddr, defaultGWMacAddr) == false)
		return -1;

	bool dhcpcdFlag = false;
	DhcpcdMgr	_dhcpcdMgr(wanNicName, "PIPMASQ");
	if ((strncmp(pseudIPAddr, "dhcp", 4) == 0) ||  (strncmp(pseudIPAddr, "DHCP", 4) == 0))
	{
		dhcpcdFlag = true;
		struct dhcpGetInfo _dhcpGetInfo; 
		_dhcpcdMgr.dhcpcdGetInfo(&_dhcpGetInfo);
		rev_inet_addr(_dhcpGetInfo.yourClientIPAddress, pseudIPAddr);
	}

	PortMgr		_portMgr;
	TableMgr	_tableMgr(&_portMgr);
	FtpMgr		_ftpMgr(&_tableMgr, &_portMgr, pseudIPAddr);
	PseudIF		_pseudIF(wanNicName, pseudIPAddr, defaultGWMacAddr);
	ArpMgr		_arpMgr(&_pseudIF);
	PingServer	_pingSvr(&_pseudIF);
	ThreadMgr	_threadMgr;

	wi.tableMgr = &_tableMgr;
	wi.portMgr = &_portMgr;
	wi.arpMgr = &_arpMgr;
	wi.pingSvr = &_pingSvr;
	wi.psuedIF = &_pseudIF;
	wi.threadMgr = &_threadMgr;
	wi.dhcpcdMgr = &_dhcpcdMgr;

	wi.pseudIPAddr = pseudIPAddr;

	char			_lanIPAddr[16];
	char			_wanIPAddr[16];

	char			_dummy_AdapterName[512];
	unsigned char	_dummy_macAddr[ETH_ALEN];
	char			_dummy_maskAddress[16];

	getAdapterInfo2(
		lanNicName, 
		_dummy_AdapterName, 
		_dummy_macAddr, 
		_lanIPAddr, 
		_dummy_maskAddress);

	wi.lanNicName = lanNicName;
	wi.lanIPAddr = _lanIPAddr;

	getAdapterInfo2(
		wanNicName, 
		_dummy_AdapterName, 
		_dummy_macAddr, 
		_wanIPAddr, 
		_dummy_maskAddress);

	wi.wanNicName = wanNicName;
	wi.wanIPAddr = _wanIPAddr;

	wi.subWanIPAddr = subWanIPAddr;
	wi.defaultWanGWAddr = defaultWanGWAddr;

	wi.pingDown = pingDown;
	wi.ftpMgr = &_ftpMgr;

	HANDLE threadHandle[3];
	unsigned threadID;

	// Start Outward thread 
	threadHandle[0] =
		(HANDLE)_beginthreadex(NULL, 0, &_outWardThread, &wi, 0, &threadID);

	// Start Inward thread 
	threadHandle[1] =
		(HANDLE)_beginthreadex(NULL, 0, &_inWardThread, &wi, 0, &threadID);

	// Start TimeMgr thread 
	threadHandle[2] =
		(HANDLE)_beginthreadex(NULL, 0, &_timeMgrThread, &wi, 0, &threadID);

#if _NTS
#else
	HANDLE threadHandleConsole;	
	unsigned threadIDConsole;	
	// Start ConsoleMgr thread
	threadHandleConsole =
		(HANDLE)_beginthreadex(NULL, 0, &_consoleMgrThread, &wi, 0, &threadIDConsole);
#endif 

	if (dhcpcdFlag == true)
		_dhcpcdMgr.dhcpcdStart(&_pseudIF);

	WaitForMultipleObjects(3, threadHandle, TRUE, INFINITE);

	for(int i = 0; i < 3; i++)
	{
		CloseHandle(threadHandle[i]);
	}

#if _NTS
#else
	// Force to finish ConsoleMgr
	TerminateThread(threadHandleConsole,0);
	CloseHandle(threadHandleConsole);
#endif 

	ReleaseMutex(hMutex);

	printf("\n\nStart Terminating.......\n");
	printf("Please wait a few seconds to terminate completely.....\n\n\n");
		
	return 0;
}


