#include "stdafx.h"
#include "PIPMasq.h"

extern CRITICAL_SECTION	critSec_PacketGetNetInfoEx;
extern CRITICAL_SECTION	critSec_getAdapterInfo2;

typedef unsigned int __u32;

// Don't forget the following next two lines before using this function
//		WSADATA			Data;
//		int status = WSAStartup(MAKEWORD(1,1), &Data);
bool
_hostnameToDotIP(const char *IPAddr, char *dotIP)
{
	memset(dotIP, 0, 16);
	
	struct in_addr iaHost;
	LPHOSTENT lpHostEntry;
	struct in_addr *pinAddr;

	iaHost.s_addr = inet_addr(IPAddr);
	if (iaHost.s_addr == INADDR_NONE)
	{
		lpHostEntry = gethostbyname(IPAddr);
		if (lpHostEntry == NULL)
			return false;

		pinAddr = (LPIN_ADDR)lpHostEntry->h_addr_list[0];

		memcpy(dotIP, inet_ntoa(*pinAddr), 16);
	}
	else
	{
		memcpy(dotIP, IPAddr, 16);		
	}

	return true;
}


// Don't forget the following next two lines before using this function
//		WSADATA			Data;
//		int status = WSAStartup(MAKEWORD(1,1), &Data);
// This function is a reverse function of inet_addr
bool
rev_inet_addr(__u32 netByteIPAddr32, char *dotIP)
{
	memset(dotIP, 0, 16);
	
	struct in_addr	addr;
	addr.s_addr = netByteIPAddr32;

	memcpy(dotIP, inet_ntoa(addr), 16); 
	return true;
}




WCHAR* SChar2WChar(char* nome)
{
	int i;
	WCHAR* TmpName;

	TmpName=(WCHAR*) malloc ((strlen(nome)+2)*sizeof(WCHAR));
	for (i=0;i<(signed)strlen(nome)+1; i++)
		TmpName[i]=nome[i];
	TmpName[i]=0;
	return TmpName;
}

//---------------------------------------------------------------------------
/* Convert a Unicode dotted-quad to a 32-bit IP address.
   Doesn't check to make sure it's valid. */

ULONG inet_addrU(const WCHAR *cp)
{
	ULONG val, part;
	WCHAR c;
	int i;

	val = 0;
	for (i = 0; i < 4; i++) {
		part = 0;
		while ((c = *cp++) != '\0' && c != '.') {
			if (c < '0' || c > '9')
				return -1;
			part = part*10 + (c - '0');
		}
		if (part > 255)
			return -1;	
		val = (val << 8) | part;
		if (i == 3) {
			if (c != '\0')
				return -1;	// extra gunk at end of string 
		} else {
			if (c == '\0')
				return -1;	// string ends early 
		}
	}
	return val;
}

BOOLEAN PacketGetNetInfoEx(
	LPTSTR AdapterName, PULONG netp, PULONG maskp, PULONG defp, bool *dhcpFlag)
{
	char	*AdapterNameA;
	WCHAR	*AdapterNameU;
	WCHAR	*ifname;
	HKEY	SystemKey;
	HKEY	ControlSetKey;
	HKEY	ServicesKey;
	HKEY	InterfaceKey;
	HKEY	ParametersKey;
	HKEY	TcpIpKey;
	LONG	status;
//	WCHAR	String[1024+1];
	DWORD	RegType;
	ULONG	BufLen;
	DWORD	DHCPEnabled;

	char	string[1024+1];
	char _charIfName[100];
	int k, ifnameLen;

	EnterCriticalSection(&critSec_PacketGetNetInfoEx);

	*dhcpFlag = false;

	AdapterNameA = (char*)AdapterName;
	if(AdapterNameA[1] != 0) {	//ASCII
		AdapterNameU = SChar2WChar(AdapterNameA);
		AdapterName = (char *)AdapterNameU;
	} else {				//Unicode
		AdapterNameU = NULL;
	}

	ifname =  wcsrchr((unsigned short *)AdapterName, '\\');
	if (ifname == NULL)
		ifname = (unsigned short *)AdapterName;
	else
		ifname++;
	
	if (wcsncmp(ifname, L"Packet_", 7) == 0)
		ifname += 7;

	status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,TEXT("SYSTEM"),0,KEY_READ,&SystemKey);
	if (status != ERROR_SUCCESS)
		goto fail;
	
	status = RegOpenKeyEx(SystemKey,TEXT("CurrentControlSet"),0,KEY_READ,&ControlSetKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(SystemKey);
		goto fail;
	}
	RegCloseKey(SystemKey);

	status = RegOpenKeyEx(ControlSetKey,TEXT("Services"),0,KEY_READ,&ServicesKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ControlSetKey);
		goto fail;
	}
	RegCloseKey(ControlSetKey);

	ifnameLen = wcslen(ifname);
	
	memset(_charIfName, 0, sizeof(_charIfName));

	for(k = 0; k < ifnameLen; k++)
	{
		_charIfName[k] = *ifname++;
	}


	status = RegOpenKeyEx(ServicesKey,_charIfName,0,KEY_READ,&InterfaceKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ServicesKey);
		goto fail;
	}
	RegCloseKey(ServicesKey);
	
	status = RegOpenKeyEx(InterfaceKey,TEXT("Parameters"),0,KEY_READ,&ParametersKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(InterfaceKey);
		goto fail;
	}
	RegCloseKey(InterfaceKey);
	
	status = RegOpenKeyEx(ParametersKey,TEXT("TcpIp"),0,KEY_READ,&TcpIpKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ParametersKey);
		goto fail;
	}
	RegCloseKey(ParametersKey);

	// IP Address
	BufLen = sizeof string;
	status = RegQueryValueEx(TcpIpKey,TEXT("IPAddress"),NULL,&RegType,(LPBYTE)string,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*netp = inet_addr(string);

	// DefaultGateway
	BufLen = sizeof string;	
	status = RegQueryValueEx(TcpIpKey,TEXT("DefaultGateway"),NULL,&RegType,(LPBYTE)string,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*defp = inet_addr(string);

	// SubnetMask
	BufLen = sizeof string;
	status=RegQueryValueEx(TcpIpKey,TEXT("SubnetMask"),NULL,&RegType,(LPBYTE)string,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*maskp = inet_addr(string);

	/* See if DHCP is used by this system */
	BufLen = 4;
	status=RegQueryValueEx(TcpIpKey,TEXT("EnableDHCP"),NULL,&RegType,(LPBYTE)&DHCPEnabled,&BufLen);
	if (status == ERROR_SUCCESS) {

		/* Retrieve the netmask */
		if(DHCPEnabled){
		
			*dhcpFlag = true;

			// DHCP IP Address
			BufLen = sizeof string;
			status=RegQueryValueEx(TcpIpKey,TEXT("DhcpIPAddress"),NULL,&RegType,
				(LPBYTE)string, &BufLen);
			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*netp = inet_addr(string);
			
			// DHCP DefaultGateway
			BufLen = sizeof string;
			status=RegQueryValueEx(TcpIpKey,TEXT("DhcpDefaultGateway"),NULL,&RegType,
				(LPBYTE)string,&BufLen);
			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*defp = inet_addr(string);
			
			// DHCP SubnetMask
			BufLen = sizeof string;
			status=RegQueryValueEx(TcpIpKey,TEXT("DHCPSubnetMask"),NULL,&RegType,
				(LPBYTE)string,&BufLen);

			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*maskp = inet_addr(string);			
		}
	}

	if (AdapterNameU != NULL)
		free(AdapterNameU);

	LeaveCriticalSection(&critSec_PacketGetNetInfoEx);
	return TRUE;


fail:
	if (AdapterNameU != NULL)
		free(AdapterNameU);

	LeaveCriticalSection(&critSec_PacketGetNetInfoEx);
    return FALSE;
}


BOOLEAN PacketGetNetInfo2(
	LPTSTR AdapterName, PULONG netp, PULONG maskp)
{
	ULONG defp;
	bool dhcpFlag;

	return PacketGetNetInfoEx(AdapterName, netp, maskp, &defp, &dhcpFlag);
}

bool getAdapterInfo2(
	const char* nicName,		// input
	char* AdapterName,			// output
	unsigned char*	macAddr,	// output	
	char* ipAddress,			// output				
	char* maskAddress			// output		
)
{
	// string that contains a list of the network adapters
	WCHAR		TempBuffer[1024]; 

	char        adapterList[512];
	char        adapterList2[512];

	char		AdapterName2[512];

	__u32		ipAddr;				// IP Address
	__u32		maskAddr;			// Mask (Subnet) Address

	EnterCriticalSection(&critSec_getAdapterInfo2);

	DWORD dwVersion=GetVersion();
	DWORD dwWindowsMajorVersion =  (DWORD)(LOBYTE(LOWORD(dwVersion)));

	// Windows NT 
	if (!(!(dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4)))
		return false;
  
	ULONG		AdapterLength = 1024;

	memset(TempBuffer,0, sizeof(TempBuffer));
	memset(adapterList,0, sizeof(adapterList));
	memset(adapterList2,0, sizeof(adapterList2));

	WCHAR		*temp,*temp1;

	int nicNameLen = strlen(nicName);
	temp = (WCHAR *)(adapterList);
	
	for(int k=0; k < nicNameLen; k++)
	{
		*temp++ = *nicName++;
	}

	// Change adapterList the part of "/" to "//" for Windows NT

	
	memcpy(adapterList2,adapterList,sizeof(adapterList));
	temp1 = temp = (WCHAR *)(adapterList2);
	
	int adapterList2Length = 0;
	while ((*temp!='\0')||(*(temp-1)!='\0'))
	{
		adapterList2Length++;
		temp++;
	}
	
	int j =0;
	temp = temp1;
	while ((*temp!='\0')||(*(temp-1)!='\0'))
	{
		if (*temp == 0x005c)
		{
			int k;
			for (k = adapterList2Length ; k > j ; k--)
			{
				*(temp1 + k + 1) = *(temp1 + k); 
			}
			
			*(temp + 1) =  0x005c;	// add "/"
			temp++;
			adapterList2Length++;
		}
		j++;
		temp++;
	}

	
	// Get the IP Address

	BOOLEAN truefalse = 
		PacketGetNetInfo2((LPTSTR)adapterList2,(PULONG)(&ipAddr),(PULONG)(&maskAddr));
	
	rev_inet_addr(ipAddr, ipAddress);
	rev_inet_addr(maskAddr, maskAddress);

	memcpy(AdapterName, adapterList, 512 /*sizeof(AdapterName) */);
	memcpy(AdapterName2, adapterList2, 512 /*sizeof(AdapterName2) */);
	
	LPADAPTER temp_lpAdapter 
		= PacketOpenAdapter((LPTSTR)adapterList);
	
	if (!temp_lpAdapter || (temp_lpAdapter->hFile == INVALID_HANDLE_VALUE))
	{
		DWORD dwErrorCode=GetLastError();
		
		return false;
	}
	
	// Check the Adapter Type (It should be Ethernet)
	NetType netType;
	
	PacketGetNetType(temp_lpAdapter, &netType);
	if (netType.LinkType != NdisMedium802_3)
		return false;
	
	// Get the MAC Address
	PACKET_OID_DATA pod;
	
	pod.Oid = OID_802_3_PERMANENT_ADDRESS;
	pod.Length = ETH_ALEN /* sizeof(macAddr) */;
	
	if (PacketRequest(temp_lpAdapter, 0, &pod) == NULL)
	{
		return false;
	}
	
	memcpy(macAddr, pod.Data, pod.Length);
	
	PacketCloseAdapter(temp_lpAdapter);

	LeaveCriticalSection(&critSec_getAdapterInfo2);
	return true;
}

